@@ -41,15 +41,16 @@ Application::Application(const QString &fileName, QApt::Backend *backend)
         , m_backend(backend)
         , m_package(0)
         , m_isValid(false)
+        , m_isTechnical(false)
 {
     m_data = desktopContents();
-//     if (!m_isValid) {
-//         kDebug() << "Not valid" << m_fileName;
-//     }
 }
 
-Application::Application(QApt::Package *package)
-        : m_package(package)
+Application::Application(QApt::Package *package, QApt::Backend *backend)
+        : m_backend(backend)
+        , m_package(package)
+        , m_isValid(true)
+        , m_isTechnical(true)
 {
 }
 
@@ -58,14 +59,31 @@ Application::~Application()
 }
 
 QString Application::name()
+{
+    return i18n(untranslatedName().toUtf8());
+}
+
+QString Application::untranslatedName()
 {
     QString name = QString::fromUtf8(getField("Name")).trimmed();
     if (name.isEmpty()) {
-        // kDebug() << m_fileName;
-        return package()->name();
+        // extras.ubuntu.com packages can have this
+        name = package()->controlField(QLatin1String("Appname"));
+
+        if (!name.isEmpty()) {
+            // Graduate to non-technical, since it has Appname
+            m_isTechnical = false;
+            return name;
+        }
+
+        if (m_isTechnical) {
+            return package()->shortDescription();
+        } else {
+            return package()->latin1Name();
+        }
     }
 
-    return i18n(name.toUtf8());
+    return name;
 }
 
 QString Application::comment()
@@ -75,10 +93,10 @@ QString Application::comment()
         // Sometimes GenericName is used instead of Comment
         comment = getField("GenericName");
         if (comment.isEmpty()) {
-            QApt::Package *pkg = package();
-
-            if (pkg) {
-                return pkg->shortDescription();
+            if (m_isTechnical) {
+                return package()->name();
+            } else {
+                return package()->shortDescription();
             }
         }
     }
@@ -116,6 +134,11 @@ QString Application::icon() const
     return icon;
 }
 
+QString Application::mimetypes() const
+{
+    return getField("MimeType");
+}
+
 QString Application::menuPath()
 {
     QString path;
@@ -201,9 +224,44 @@ QVector<QPair<QString, QString> > Application::locateApplication(const QString &
     return ret;
 }
 
-QString Application::categories() const
+QString Application::categories()
 {
-    return getField("Categories");
+    QString categories = getField("Categories");
+
+    if (categories.isEmpty()) {
+        // extras.ubuntu.com packages can have this field
+        categories = package()->controlField(QLatin1String("Category"));
+
+        if (!categories.isEmpty()) {
+            // Graduate to non-technical, since it has Appname
+            m_isTechnical = false;
+        }
+    }
+    return categories;
+}
+
+KUrl Application::screenshotUrl(QApt::ScreenshotType type)
+{
+    QString appUrl;
+    switch (type) {
+    case QApt::Thumbnail:
+        appUrl = package()->controlField(QLatin1String("Thumbnail-Url"));
+        break;
+    case QApt::Screenshot:
+        appUrl = package()->controlField(QLatin1String("Screenshot-Url"));
+        break;
+    default:
+        break;
+    }
+
+    KUrl url;
+    if (appUrl.isEmpty()) {
+        url = package()->screenshotUrl(type);
+    } else {
+        url = KUrl(appUrl);
+    }
+
+    return url;
 }
 
 QApt::PackageList Application::addons()
@@ -264,18 +322,16 @@ QApt::PackageList Application::addons()
     return addons;
 }
 
-int Application::popconScore() const
-{
-    QString popconString = getField("X-AppInstall-Popcon");
-
-    return popconString.toInt();
-}
-
 bool Application::isValid() const
 {
     return m_isValid;
 }
 
+bool Application::isTechnical() const
+{
+    return m_isTechnical;
+}
+
 QByteArray Application::getField(const QByteArray &field) const
 {
     return m_data.value(field);
@@ -319,9 +375,7 @@ QHash<QByteArray, QByteArray> Application::desktopContents()
             aKey = line.left(eqpos);
             aValue = line.right(line.size() - eqpos -1);
 
-            if (!contents.contains(aKey)) {
-                contents[aKey] = aValue;
-            }
+            contents[aKey] = aValue;
         }
 
         lineIndex++;
