@@ -34,6 +34,7 @@
 
 #include "Application.h"
 #include "ApplicationLauncher.h"
+#include "ReviewsBackend/ReviewsBackend.h"
 #include "Transaction.h"
 
 ApplicationBackend::ApplicationBackend(QObject *parent)
@@ -44,6 +45,8 @@ ApplicationBackend::ApplicationBackend(QObject *parent)
     m_currentTransaction = m_queue.end();
 
     m_pkgBlacklist << "kdebase-runtime" << "kdepim-runtime" << "kdelibs5-plugins";
+
+    m_reviewsBackend = new ReviewsBackend(this);
 }
 
 ApplicationBackend::~ApplicationBackend()
@@ -66,25 +69,38 @@ void ApplicationBackend::setBackend(QApt::Backend *backend)
 
 void ApplicationBackend::init()
 {
-    QList<int> popconScores;
+    m_reviewsBackend->setAptBackend(m_backend);
+
     QDir appDir("/usr/share/app-install/desktop/");
     QStringList fileList = appDir.entryList(QDir::Files);
+
+    QList<Application *> tempList;
     foreach(const QString &fileName, fileList) {
         Application *app = new Application("/usr/share/app-install/desktop/" + fileName, m_backend);
+        tempList << app;
+    }
+
+    foreach (QApt::Package *package, m_backend->availablePackages()) {
+        Application *app = new Application(package, m_backend);
+        tempList << app;
+    }
+
+    foreach (Application *app, tempList) {
         if (app->isValid()) {
-            if (app->package() && !m_pkgBlacklist.contains(app->package()->latin1Name())) {
+            QApt::Package *pkg = app->package();
+            if ((pkg) && !m_pkgBlacklist.contains(pkg->latin1Name())) {
                 m_appList << app;
-                if (app->package()->isInstalled()) {
-                    m_instOriginList << app->package()->origin();
-                    m_originList << app->package()->origin();
+                if (pkg->isInstalled()) {
+                    m_instOriginList << pkg->origin();
+                    m_originList << pkg->origin();
                 } else {
-                    m_originList << app->package()->origin();
+                    m_originList << pkg->origin();
                 }
-                popconScores << app->popconScore();
+            } else {
+                delete app;
             }
         } else {
             // Invalid .desktop file
-            // kDebug() << fileName;
             delete app;
         }
     }
@@ -96,10 +112,6 @@ void ApplicationBackend::init()
     if (m_instOriginList.contains(QLatin1String(""))) {
         m_instOriginList.remove(QLatin1String(""));
     }
-
-    qSort(popconScores);
-
-    m_maxPopconScore = popconScores.last();
 }
 
 void ApplicationBackend::reload()
@@ -110,6 +122,7 @@ void ApplicationBackend::reload()
     qDeleteAll(m_queue);
     m_queue.clear();
     m_backend->reloadCache();
+    m_reviewsBackend->clearReviewCache();
 
     init();
 
@@ -370,6 +383,11 @@ void ApplicationBackend::onAppLauncherClosed()
     m_appLauncher = 0;
 }
 
+ReviewsBackend *ApplicationBackend::reviewsBackend() const
+{
+    return m_reviewsBackend;
+}
+
 QList<Application *> ApplicationBackend::applicationList() const
 {
     return m_appList;
@@ -394,8 +412,3 @@ QList<Transaction *> ApplicationBackend::transactions() const
 {
     return m_queue;
 }
-
-int ApplicationBackend::maxPopconScore() const
-{
-    return m_maxPopconScore;
-}
