@@ -25,12 +25,11 @@
 #include <QtCore/QStringBuilder>
 #include <QtGui/QHBoxLayout>
 #include <QtGui/QLabel>
-#include <QtGui/QListView>
 #include <QtGui/QProgressBar>
 #include <QtGui/QPushButton>
-#include <QtGui/QStandardItemModel>
 #include <QtGui/QVBoxLayout>
 
+#include <KDialog>
 #include <KGlobal>
 #include <KHBox>
 #include <KIcon>
@@ -39,6 +38,7 @@
 #include <KLocale>
 #include <KPixmapSequence>
 #include <kpixmapsequenceoverlaypainter.h>
+#include <KService>
 #include <KStandardGuiItem>
 #include <KTemporaryFile>
 #include <KToolInvocation>
@@ -47,12 +47,28 @@
 
 #include <LibQApt/Package>
 
+
+#include "HaveQZeitgeist.h"
+#ifdef HAVE_QZEITGEIST
+#include <QtZeitgeist/DataModel/Event>
+#include <QtZeitgeist/DataModel/TimeRange>
+#include <QtZeitgeist/Log>
+#include <QtZeitgeist/Interpretation>
+#include <QtZeitgeist/Manifestation>
+#include <QtZeitgeist/QtZeitgeist>
+#endif
+
 #include <math.h>
 
+#include "AddonsWidget.h"
 #include "Application.h"
 #include "ClickableLabel.h"
 #include "effects/GraphicsOpacityDropShadowEffect.h"
 #include "ScreenShotViewer.h"
+#include "ReviewsBackend/Rating.h"
+#include "ReviewsBackend/Review.h"
+#include "ReviewsBackend/ReviewsWidget.h"
+#include "ReviewsBackend/ReviewsBackend.h"
 #include "../../libmuon/MuonStrings.h"
 
 #define BLUR_RADIUS 15
@@ -62,13 +78,15 @@ ApplicationDetailsWidget::ApplicationDetailsWidget(QWidget *parent, ApplicationB
     , m_appBackend(backend)
     , m_screenshotFile(0)
     , m_screenshotDialog(0)
-    , m_strings(new MuonStrings(this))
 {
     setFrameShape(QFrame::NoFrame);
     setWidgetResizable(true);
     viewport()->setAutoFillBackground(false);
 
     QWidget *widget = new QWidget(this);
+    setStyleSheet(QString("ApplicationDetailsWidget {"
+                          "background-color: rgb(255, 255, 255);"
+                          "}"));
     QVBoxLayout *layout = new QVBoxLayout(widget);
     widget->setLayout(layout);
 
@@ -91,14 +109,23 @@ ApplicationDetailsWidget::ApplicationDetailsWidget(QWidget *parent, ApplicationB
     QWidget *headerSpacer = new QWidget(headerWidget);
     headerSpacer->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
 
-    m_ratingWidget = new KRatingWidget(headerWidget);
+    QWidget *ratingUseWidget = new QWidget(headerWidget);
+    QVBoxLayout *ratingUseLayout = new QVBoxLayout(ratingUseWidget);
+
+    m_ratingWidget = new KRatingWidget(ratingUseWidget);
     m_ratingWidget->setAttribute(Qt::WA_TransparentForMouseEvents);
     m_ratingWidget->setPixmapSize(32);
 
+    m_ratingCountLabel = new QLabel(ratingUseWidget);
+    m_ratingCountLabel->setAlignment(Qt::AlignHCenter);
+
+    ratingUseLayout->addWidget(m_ratingWidget);
+    ratingUseLayout->addWidget(m_ratingCountLabel);
+
     headerLayout->addWidget(m_iconLabel);
     headerLayout->addWidget(nameDescWidget);
     headerLayout->addWidget(headerSpacer);
-    headerLayout->addWidget(m_ratingWidget);
+    headerLayout->addWidget(ratingUseWidget);
 
     // Menu path label
     m_menuPathWidget = new QWidget(this);
@@ -124,6 +151,7 @@ ApplicationDetailsWidget::ApplicationDetailsWidget(QWidget *parent, ApplicationB
     actionButtonWidget->setFrameShape(QFrame::StyledPanel);
 
     m_statusLabel = new QLabel(actionButtonWidget);
+    m_usageLabel = new QLabel(actionButtonWidget);
 
     QWidget *actionButtonSpacer = new QWidget(actionButtonWidget);
     actionButtonSpacer->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
@@ -143,6 +171,7 @@ ApplicationDetailsWidget::ApplicationDetailsWidget(QWidget *parent, ApplicationB
     connect(m_cancelButton, SIGNAL(clicked()), this, SLOT(cancelButtonClicked()));
 
     actionButtonLayout->addWidget(m_statusLabel);
+    actionButtonLayout->addWidget(m_usageLabel);
     actionButtonLayout->addWidget(actionButtonSpacer);
     actionButtonLayout->addWidget(m_actionButton);
     actionButtonLayout->addWidget(m_progressBar);
@@ -152,11 +181,18 @@ ApplicationDetailsWidget::ApplicationDetailsWidget(QWidget *parent, ApplicationB
     QWidget *body = new QWidget(widget);
     QHBoxLayout *bodyLayout = new QHBoxLayout(body);
 
-    m_longDescLabel = new QLabel(body);
+    KVBox *bodyLeft = new KVBox(body);
+    bodyLeft->setSpacing(2*KDialog::spacingHint());
+
+    m_longDescLabel = new QLabel(bodyLeft);
     m_longDescLabel->setWordWrap(true);
     m_longDescLabel->setAlignment(Qt::AlignLeft | Qt::AlignTop);
     m_longDescLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
 
+    m_websiteLabel = new QLabel(bodyLeft);
+    m_websiteLabel->setAlignment(Qt::AlignLeft);
+    m_websiteLabel->setOpenExternalLinks(true);
+
     m_screenshotLabel = new ClickableLabel(body);
     m_screenshotLabel->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
     m_screenshotLabel->setMinimumSize(170, 130);
@@ -169,88 +205,50 @@ ApplicationDetailsWidget::ApplicationDetailsWidget(QWidget *parent, ApplicationB
 
     connect(m_screenshotLabel, SIGNAL(clicked()), this, SLOT(screenshotLabelClicked()));
 
-    bodyLayout->addWidget(m_longDescLabel);
+    bodyLayout->addWidget(bodyLeft);
     bodyLayout->addWidget(m_screenshotLabel);
 
-    m_addonsWidget = new QWidget(widget);
-    QVBoxLayout *addonsLayout = new QVBoxLayout(m_addonsWidget);
-    m_addonsWidget->setLayout(addonsLayout);
+    m_addonsWidget = new AddonsWidget(widget, m_appBackend);
+    connect(m_addonsWidget, SIGNAL(applyButtonClicked(QHash<QApt::Package *, QApt::Package::State>)),
+            this, SLOT(addonsApplyButtonClicked(QHash<QApt::Package *, QApt::Package::State>)));
     m_addonsWidget->hide();
 
-    QLabel *addonsTitle = new QLabel(m_addonsWidget);
-    addonsTitle->setText(i18nc("@title:group", "Addons"));
-    addonsTitle->setAlignment(Qt::AlignLeft);
-
-    m_addonsView = new QListView(m_addonsWidget);
-    m_addonsView->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Maximum);
-    m_addonsModel = new QStandardItemModel(this);
-    m_addonsView->setModel(m_addonsModel);
-
-    QWidget *addonsButtonBox = new QWidget(m_addonsWidget);
-    QHBoxLayout *addonButtonsLayout = new QHBoxLayout(addonsButtonBox);
-
-    QWidget *addonsButtonSpacer = new QWidget(m_addonsWidget);
-    addonsButtonSpacer->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
-
-    m_addonsRevertButton = new QPushButton(addonsButtonBox);
-    m_addonsRevertButton->setIcon(KIcon("edit-undo"));
-    m_addonsRevertButton->setText(i18nc("@action:button", "Revert"));
-    connect(m_addonsRevertButton, SIGNAL(clicked()),
-            this, SLOT(addonsRevertButtonClicked()));
-
-    m_addonsApplyButton = new QPushButton(m_addonsWidget);
-    m_addonsApplyButton->setIcon(KIcon("dialog-ok-apply"));
-    m_addonsApplyButton->setText(i18nc("@action:button", "Apply"));
-    m_addonsApplyButton->setToolTip(i18nc("@info:tooltip", "Apply changes to addons"));
-    connect(m_addonsApplyButton, SIGNAL(clicked()),
-            this, SLOT(addonsApplyButtonClicked()));
-
-    addonButtonsLayout->addWidget(addonsButtonSpacer);
-    addonButtonsLayout->addWidget(m_addonsRevertButton);
-    addonButtonsLayout->addWidget(m_addonsApplyButton);
-
-    addonsLayout->addWidget(addonsTitle);
-    addonsLayout->addWidget(m_addonsView);
-    addonsLayout->addWidget(addonsButtonBox);
-
-    m_websiteLabel = new QLabel(this);
-    m_websiteLabel->setAlignment(Qt::AlignLeft);
-    m_websiteLabel->setOpenExternalLinks(true);
-
     // Technical details
     QWidget *detailsWidget = new QWidget(widget);
     QGridLayout *detailsGrid = new QGridLayout(detailsWidget);
 
     // detailsGrid, row 0
     QLabel *sizeLabel = new QLabel(detailsWidget);
-    sizeLabel->setText(i18nc("@label Label preceding the app size", "Total Size:"));
+    sizeLabel->setText("<b>" % i18nc("@label Label preceding the app size", "Total Size:")  % "</b>");
     m_size = new QLabel(detailsWidget);
     detailsGrid->addWidget(sizeLabel, 0, 0, Qt::AlignRight);
     detailsGrid->addWidget(m_size, 0, 1, Qt::AlignLeft);
 
     // detailsGrid, row 1
     QLabel *versionLabel = new QLabel(detailsWidget);
-    versionLabel->setText(i18nc("@label Label preceding the app version", "Version:"));
+    versionLabel->setText("<b>" % i18nc("@label/rich Label preceding the app version", "Version:") % "</b>");
     m_version = new QLabel(detailsWidget);
     detailsGrid->addWidget(versionLabel, 1, 0, Qt::AlignRight);
     detailsGrid->addWidget(m_version, 1, 1, Qt::AlignLeft);
 
     // detailsGrid, row 2
     QLabel *licenseLabel = new QLabel(detailsWidget);
-    licenseLabel->setText(i18nc("@label Label preceding the app license", "License:"));
+    licenseLabel->setText("<b>" % i18nc("@label Label preceding the app license", "License:") % "</b>");
     m_license = new QLabel(detailsWidget);
     detailsGrid->addWidget(licenseLabel, 2, 0, Qt::AlignRight);
     detailsGrid->addWidget(m_license, 2, 1, Qt::AlignLeft);
 
     // detailsGrid, row 3
     QLabel *supportLabel = new QLabel(detailsWidget);
-    supportLabel->setText(i18nc("@label Label preceding the app support", "Support:"));
+    supportLabel->setText("<b>" % i18nc("@label Label preceding the app support", "Support:") % "</b>");
     m_support = new QLabel(detailsWidget);
     detailsGrid->addWidget(supportLabel, 3, 0, Qt::AlignRight);
     detailsGrid->addWidget(m_support, 3, 1, Qt::AlignLeft);
 
     detailsGrid->setColumnStretch(1,1);
 
+    m_reviewsWidget = new ReviewsWidget(widget);
+
     QWidget *verticalSpacer = new QWidget(this);
     verticalSpacer->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Expanding);
 
@@ -260,8 +258,8 @@ ApplicationDetailsWidget::ApplicationDetailsWidget(QWidget *parent, ApplicationB
     layout->addWidget(actionButtonWidget);
     layout->addWidget(body);
     layout->addWidget(m_addonsWidget);
-    layout->addWidget(m_websiteLabel);
     layout->addWidget(detailsWidget);
+    layout->addWidget(m_reviewsWidget);
     layout->addWidget(verticalSpacer);
 
     connect(m_appBackend, SIGNAL(workerEvent(QApt::WorkerEvent, Transaction *)),
@@ -288,7 +286,19 @@ void ApplicationDetailsWidget::setApplication(Application *app)
     m_nameLabel->setText(QLatin1Literal("<h1>") % app->name() % QLatin1Literal("</h1>"));
     m_shortDescLabel->setText(app->comment());
 
-    m_ratingWidget->setRating((int)(10* log(app->popconScore())/log(m_appBackend->maxPopconScore()+1)));
+    ReviewsBackend *reviewsBackend = m_appBackend->reviewsBackend();
+    Rating *rating = reviewsBackend->ratingForApplication(app);
+    if (rating) {
+        m_ratingWidget->setRating(rating->rating());
+        m_ratingCountLabel->setText(i18ncp("@label The number of ratings the app has",
+                                           "%1 rating", "%1 ratings",
+                                           rating->ratingCount()));
+    } else {
+        m_ratingWidget->hide();
+        m_ratingCountLabel->hide();
+    }
+
+    populateZeitgeistInfo();
 
     QString menuPathString = app->menuPath();
     if (!menuPathString.isEmpty()) {
@@ -298,12 +308,12 @@ void ApplicationDetailsWidget::setApplication(Application *app)
     }
 
     if (!app->package()->isInstalled()) {
-        m_statusLabel->setText(m_strings->packageStateName(QApt::Package::NotInstalled));
+        m_statusLabel->setText(MuonStrings::global()->packageStateName(QApt::Package::NotInstalled));
         m_actionButton->setText(i18nc("@action", "Install"));
         m_actionButton->setIcon(KIcon("download"));
         m_actionButton->show();
     } else {
-        m_statusLabel->setText(m_strings->packageStateName(QApt::Package::Installed));
+        m_statusLabel->setText(MuonStrings::global()->packageStateName(QApt::Package::Installed));
         m_actionButton->setText(i18nc("@action", "Remove"));
         m_actionButton->setIcon(KIcon("edit-delete"));
     }
@@ -342,6 +352,8 @@ void ApplicationDetailsWidget::setApplication(Application *app)
         m_license->setText(i18nc("@info license", "Open Source"));
     } else if (app->package()->component() == "restricted") {
         m_license->setText(i18nc("@info license", "Proprietary"));
+    } else {
+        m_license->setText(i18nc("@info license", "Unknown"));
     }
 
     if (app->package()->isSupported()) {
@@ -354,6 +366,11 @@ void ApplicationDetailsWidget::setApplication(Application *app)
                                  "may be provided by the Ubuntu community", app->name()));
     }
 
+    // Fetch reviews
+    reviewsBackend->fetchReviews(app);
+    connect(reviewsBackend, SIGNAL(reviewsReady(Application *, QList<Review *>)),
+            this, SLOT(populateReviews(Application *, QList<Review *>)));
+
     // Catch already-begun downloads. If the state is something else, we won't
     // care because we won't handle it
     QPair<QApt::WorkerEvent, Transaction *> workerState = m_appBackend->workerState();
@@ -473,6 +490,73 @@ void ApplicationDetailsWidget::transactionCancelled(Application *app)
     }
 }
 
+void ApplicationDetailsWidget::populateZeitgeistInfo()
+{
+#ifdef HAVE_QZEITGEIST
+    if (!m_app->package()->isInstalled()) {
+        m_usageLabel->hide();
+        return;
+    }
+
+    QString desktopFile;
+
+    foreach (const QString &desktop, m_app->package()->installedFilesList().filter(".desktop")) {
+            KService::Ptr service = KService::serviceByDesktopPath(desktop);
+            if (!service) {
+                break;
+            }
+
+            if (service->isApplication() &&
+              !service->noDisplay() &&
+              !service->exec().isEmpty())
+            {
+                desktopFile = desktop.split('/').last();
+                break;
+            }
+    }
+
+    QtZeitgeist::init();
+
+    // Prepare the Zeitgeist query
+    QtZeitgeist::DataModel::EventList eventListTemplate;
+
+    QtZeitgeist::DataModel::Event event1;
+    event1.setActor("application://" + desktopFile);
+    event1.setInterpretation(QtZeitgeist::Interpretation::Event::ZGModifyEvent);
+    event1.setManifestation(QtZeitgeist::Manifestation::Event::ZGUserActivity);
+
+    QtZeitgeist::DataModel::Event event2;
+    event2.setActor("application://" + desktopFile);
+    event2.setInterpretation(QtZeitgeist::Interpretation::Event::ZGCreateEvent);
+    event2.setManifestation(QtZeitgeist::Manifestation::Event::ZGUserActivity);
+
+    eventListTemplate << event1 << event2;
+
+    QtZeitgeist::Log log;
+    QDBusPendingReply<QtZeitgeist::DataModel::EventIdList> reply = log.findEventIds(QtZeitgeist::DataModel::TimeRange::always(),
+                                                                                    eventListTemplate,
+                                                                                    QtZeitgeist::Log::Any,
+                                                                                    0, QtZeitgeist::Log::MostRecentEvents);
+    reply.waitForFinished();
+
+    int usageCount = 0;
+    if (reply.isValid()) {
+        usageCount = reply.value().size();
+    }
+
+    if (!usageCount) {
+        // More likely there are no stats than no usage, so just hide the label and return
+        m_usageLabel->hide();
+        return;
+    }
+
+    m_usageLabel->setText(i18ncp("@label The number of times an app has been used",
+                                  "Used one time", "(Used %1 times)", usageCount));
+#else
+    m_usageLabel->hide();
+#endif // HAVE_QZEITGEIST
+}
+
 void ApplicationDetailsWidget::fadeInScreenshot()
 {
     m_fadeScreenshot->setDirection(QAbstractAnimation::Forward);
@@ -491,7 +575,7 @@ void ApplicationDetailsWidget::fetchScreenshot(QApt::ScreenshotType screenshotTy
     m_screenshotFile->setSuffix(".png");
     m_screenshotFile->open();
 
-    KIO::FileCopyJob *getJob = KIO::file_copy(m_app->package()->screenshotUrl(screenshotType),
+    KIO::FileCopyJob *getJob = KIO::file_copy(m_app->screenshotUrl(screenshotType),
                                m_screenshotFile->fileName(), -1, KIO::Overwrite | KIO::HideProgressInfo);
 
     switch (screenshotType) {
@@ -586,110 +670,25 @@ void ApplicationDetailsWidget::populateAddons()
 {
     QApt::PackageList addons = m_app->addons();
 
-    m_availableAddons = addons;
-
     if (!addons.isEmpty()) {
+        m_addonsWidget->setAddons(addons);
         m_addonsWidget->show();
-        connect(m_addonsModel, SIGNAL(dataChanged(const QModelIndex &, const QModelIndex &)),
-                this, SLOT(addonStateChanged(const QModelIndex &, const QModelIndex &)));
     }
-
-    foreach (QApt::Package *addon, addons) {
-        // Check if we have an application for the addon
-        Application *addonApp = 0;
-
-        foreach (Application *app, m_appBackend->applicationList()) {
-            if (app->package()->name() == addon->name()) {
-                addonApp = app;
-            }
-        }
-
-        QStandardItem *addonItem = new QStandardItem;
-        addonItem->setData(addon->latin1Name());
-        QString packageName = QLatin1Literal(" (") % addon->latin1Name() % ')';
-        if (addonApp) {
-            addonItem->setText(addonApp->name() % packageName);
-            addonItem->setIcon(KIcon(addonApp->icon()));
-        } else {
-            addonItem->setText(addon->shortDescription() % packageName);
-            addonItem->setIcon(KIcon("applications-other"));
-        }
-
-        addonItem->setEditable(false);
-        addonItem->setCheckable(true);
-
-        if (addon->isInstalled()) {
-            addonItem->setCheckState(Qt::Checked);
-        } else {
-            addonItem->setCheckState(Qt::Unchecked);
-        }
-
-        m_addonsModel->appendRow(addonItem);
-    }
-
-    m_addonsRevertButton->setEnabled(false);
-    m_addonsApplyButton->setEnabled(false);
 }
 
-void ApplicationDetailsWidget::addonStateChanged(const QModelIndex &left, const QModelIndex &right)
+void ApplicationDetailsWidget::populateReviews(Application *app, const QList<Review *> &reviews)
 {
-    Q_UNUSED(right);
-    QStandardItem *item = m_addonsModel->itemFromIndex(left);
-    QApt::Package *addon = 0;
-
-    QString addonName = item->data().toString();
-    foreach(QApt::Package *pkg, m_availableAddons) {
-        if (pkg->latin1Name() == addonName) {
-            addon = pkg;
-        }
-    }
-
-    if (!addon) {
+    if (app != m_app) {
         return;
     }
 
-    if (addon->isInstalled()) {
-        switch (item->checkState()) {
-        case Qt::Checked:
-            if (m_changedAddons.contains(addon)) {
-                m_changedAddons.remove(addon);
-            }
-            break;
-        case Qt::Unchecked:
-            m_changedAddons[addon] = QApt::Package::ToRemove;
-            break;
-        default:
-            break;
-        }
-    } else {
-        switch (item->checkState()) {
-        case Qt::Checked:
-            m_changedAddons[addon] = QApt::Package::ToInstall;
-            break;
-        case Qt::Unchecked:
-            if (m_changedAddons.contains(addon)) {
-                m_changedAddons.remove(addon);
-            }
-            break;
-        default:
-            break;
-        }
-    }
-
-    m_addonsRevertButton->setEnabled(!m_changedAddons.isEmpty());
-    m_addonsApplyButton->setEnabled(!m_changedAddons.isEmpty());
+    m_reviewsWidget->addReviews(reviews);
 }
 
-void ApplicationDetailsWidget::addonsRevertButtonClicked()
+void ApplicationDetailsWidget::addonsApplyButtonClicked(const QHash<QApt::Package *,
+                                                        QApt::Package::State> &changedAddons)
 {
-    m_availableAddons.clear();
-    m_addonsModel->clear();
-    populateAddons();
-}
-
-void ApplicationDetailsWidget::addonsApplyButtonClicked()
-{
-    emit installButtonClicked(m_app, m_changedAddons);
+    emit installButtonClicked(m_app, changedAddons);
 
     m_actionButton->hide();
     m_progressBar->show();
