@@ -30,7 +30,7 @@
 static const char description[] =
     I18N_NOOP("An application manager");
 
-static const char version[] = "1.1.3 \"Bullish Bandril\"";
+static const char version[] = "1.1.65 \"Caustic Carrionite\"";
 
 int main(int argc, char **argv)
 {
@@ -38,7 +38,6 @@ int main(int argc, char **argv)
                      KAboutData::License_GPL, ki18n("©2010, 2011 Jonathan Thomas"), KLocalizedString(), 0);
     about.addAuthor(ki18n("Jonathan Thomas"), KLocalizedString(), "echidnaman@kubuntu.org");
     about.setProgramIconName("applications-other");
-    about.setProductName("muon/installer");
 
     KCmdLineArgs::init(argc, argv, &about);
 
