@@ -36,6 +36,7 @@
 
 // LibQApt includes
 #include <LibQApt/Backend>
+#include <LibQApt/DebFile>
 
 MuonMainWindow::MuonMainWindow()
     : KXmlGuiWindow(0)
@@ -167,9 +168,7 @@ void MuonMainWindow::networkChanged()
 
 bool MuonMainWindow::isConnected() {
     int status = Solid::Networking::status();
-    bool connected = ((status == Solid::Networking::Connected) ||
-                      (status == Solid::Networking::Unknown));
-    return connected;
+    return status & (Solid::Networking::Connected | Solid::Networking::Unknown);
 }
 
 void MuonMainWindow::checkForUpdates()
@@ -180,13 +179,13 @@ void MuonMainWindow::workerEvent(QApt::WorkerEvent event)
 {
     switch (event) {
     case QApt::CacheUpdateStarted:
-        break;
     case QApt::PackageDownloadStarted:
         m_powerInhibitor = Solid::PowerManagement::beginSuppressingSleep(i18nc("@info:status", "Muon is making system changes"));
     case QApt::CommitChangesStarted:
         m_canExit = false;
         break;
     case QApt::CacheUpdateFinished:
+    case QApt::PackageDownloadFinished:
     case QApt::CommitChangesFinished:
         Solid::PowerManagement::stopSuppressingSleep(m_powerInhibitor);
         m_canExit = true;
@@ -226,7 +225,6 @@ void MuonMainWindow::errorOccurred(QApt::ErrorCode code, const QVariantMap &args
                      "any packages.");
         title = i18nc("@title:window", "Unable to obtain package system lock");
         KMessageBox::error(this, text, title);
-        setActionsEnabled();
         break;
     case QApt::DiskSpaceError: {
         QString drive = args["DirectoryString"].toString();
@@ -235,8 +233,6 @@ void MuonMainWindow::errorOccurred(QApt::ErrorCode code, const QVariantMap &args
                      "at %1 to continue with this operation.", drive);
         title = i18nc("@title:window", "Low disk space");
         KMessageBox::error(this, text, title);
-        setActionsEnabled();
-        break;
     }
     case QApt::FetchError:
         text = i18nc("@label",
@@ -246,7 +242,7 @@ void MuonMainWindow::errorOccurred(QApt::ErrorCode code, const QVariantMap &args
         break;
     case QApt::CommitError: {
         m_errorStack.append(args);
-        return;
+        break;
     }
     case QApt::AuthError:
         text = i18nc("@label",
@@ -278,15 +274,15 @@ void MuonMainWindow::errorOccurred(QApt::ErrorCode code, const QVariantMap &args
         }
         title = i18nc("@title:window", "Untrusted Packages");
         KMessageBox::errorList(this, text, untrustedItems, title);
-        setActionsEnabled();
         break;
     }
     case QApt::UserCancelError:
     case QApt::UnknownError:
     default:
-        setActionsEnabled();
         break;
     }
+
+    setActionsEnabled();
     m_canExit = true; // If we were committing changes, we aren't anymore
 }
 
@@ -393,7 +389,6 @@ void MuonMainWindow::showQueuedErrors()
 
     QString title = i18nc("@title:window", "Commit error");
     KMessageBox::detailedError(this, text, details, title);
-    m_canExit = true;
 }
 
 void MuonMainWindow::reload()
@@ -450,11 +445,49 @@ bool MuonMainWindow::saveInstalledPackagesList()
     return true;
 }
 
-void MuonMainWindow::loadSelections()
+bool MuonMainWindow::createDownloadList()
 {
     QString filename;
+    filename = KFileDialog::getSaveFileName(QString(), QString(), this,
+                                            i18nc("@title:window", "Save Download List As"));
 
-    filename = KFileDialog::getOpenFileName(QString(), QString(), this, i18nc("@title:window", "Open File"));
+    if (filename.isEmpty()) {
+        return false;
+    }
+
+    if (!m_backend->saveDownloadList(filename)) {
+        QString text = i18nc("@label", "The document could not be saved, as it "
+                             "was not possible to write to "
+                             "<filename>%1</filename>\n\nCheck "
+                             "that you have write access to this file "
+                             "or that enough disk space is available.",
+                             filename);
+        KMessageBox::error(this, text, QString());
+        return false;
+    }
+
+    return true;
+}
+
+void MuonMainWindow::downloadPackagesFromList()
+{
+    QString filename = KFileDialog::getOpenFileName(QString(), QString(),
+                                                    this, i18nc("@title:window", "Open File"));
+
+    if (filename.isEmpty()) {
+        return;
+    }
+
+    QString dirName = filename.left(filename.lastIndexOf('/'));
+
+    setActionsEnabled(false);
+    m_backend->downloadArchives(filename, dirName % QLatin1String("/packages"));
+}
+
+void MuonMainWindow::loadSelections()
+{
+    QString filename = KFileDialog::getOpenFileName(QString(), QString(),
+                                                    this, i18nc("@title:window", "Open File"));
 
     if (filename.isEmpty()) {
         return;
@@ -470,6 +503,49 @@ void MuonMainWindow::loadSelections()
     }
 }
 
+void MuonMainWindow::loadArchives()
+{
+    QString dirName;
+
+    dirName = KFileDialog::getExistingDirectory(KUrl(), this,
+                                                i18nc("@title:window",
+                                                      "Choose a Directory"));
+
+    if (dirName.isEmpty()) {
+        // User cancelled
+        return;
+    }
+
+    QDir dir(dirName);
+    QStringList archiveFiles = dir.entryList(QDir::Files, QDir::Name);
+
+    int successCount = 0;
+    foreach (const QString &archiveFile, archiveFiles) {
+        const QApt::DebFile debFile(dirName % '/' % archiveFile);
+
+        if (debFile.isValid()) {
+            if (m_backend->addArchiveToCache(debFile)) {
+                successCount++;
+            }
+        }
+    }
+
+    if (successCount) {
+        QString message = i18ncp("@label",
+                                 "%1 package was successfully added to the cache",
+                                 "%1 packages were successfully added to the cache",
+                                 successCount);
+        KMessageBox::information(this, message, QString());
+    } else {
+        QString message = i18nc("@label",
+                                "No valid packages could be found in this directory. "
+                                "Please make sure the packages are compatible with your "
+                                "computer and are at the latest version.");
+        KMessageBox::error(this, message, i18nc("@title:window",
+                                                "Packages Could Not be Found"));
+    }
+}
+
 void MuonMainWindow::undo()
 {
     m_backend->undo();
