@@ -49,6 +49,7 @@
 // Own includes
 #include "config/ManagerSettingsDialog.h"
 #include "../libmuon/CommitWidget.h"
+#include "../libmuon/HistoryView/HistoryView.h"
 #include "../libmuon/DownloadWidget.h"
 #include "../libmuon/StatusWidget.h"
 #include "FilterWidget.h"
@@ -60,6 +61,7 @@ MainWindow::MainWindow()
     : MuonMainWindow()
     , m_stack(0)
     , m_settingsDialog(0)
+    , m_historyDialog(0)
     , m_reviewWidget(0)
     , m_downloadWidget(0)
     , m_commitWidget(0)
@@ -159,6 +161,25 @@ void MainWindow::setupActions()
     m_saveSelectionsAction->setText(i18nc("@action", "Save Markings As..."));
     connect(m_saveSelectionsAction, SIGNAL(triggered()), this, SLOT(saveSelections()));
 
+    m_createDownloadListAction = actionCollection()->addAction("save_download_list");
+    m_createDownloadListAction->setIcon(KIcon("document-save-as"));
+    m_createDownloadListAction->setText(i18nc("@action", "Save Package Download List..."));
+    connect(m_createDownloadListAction, SIGNAL(triggered()), this, SLOT(createDownloadList()));
+
+    m_downloadListAction = actionCollection()->addAction("download_from_list");
+    m_downloadListAction->setIcon(KIcon("download"));
+    m_downloadListAction->setText(i18nc("@action", "Download Packages From List..."));
+    connect(m_downloadListAction, SIGNAL(triggered()), this, SLOT(downloadPackagesFromList()));
+    if (!isConnected()) {
+        m_downloadListAction->setDisabled(false);
+    }
+    connect(this, SIGNAL(shouldConnect(bool)), m_downloadListAction, SLOT(setEnabled(bool)));
+
+    m_loadArchivesAction = actionCollection()->addAction("load_archives");
+    m_loadArchivesAction->setIcon(KIcon("document-open"));
+    m_loadArchivesAction->setText(i18nc("@action", "Add Downloaded Packages"));
+    connect(m_loadArchivesAction, SIGNAL(triggered()), this, SLOT(loadArchives()));
+
     m_saveInstalledAction = actionCollection()->addAction("save_package_list");
     m_saveInstalledAction->setIcon(KIcon("document-save-as"));
     m_saveInstalledAction->setText(i18nc("@action", "Save Installed Packages List..."));
@@ -190,11 +211,15 @@ void MainWindow::setupActions()
     m_applyAction->setIcon(KIcon("dialog-ok-apply"));
     m_applyAction->setText(i18nc("@action Applys the changes a user has made", "Apply Changes"));
     connect(m_applyAction, SIGNAL(triggered()), this, SLOT(startCommit()));
-    m_applyAction->setEnabled(isConnected());
-    connect(this, SIGNAL(shouldConnect(bool)), m_applyAction, SLOT(setEnabled(bool)));
 
     KStandardAction::preferences(this, SLOT(editSettings()), actionCollection());
 
+    m_historyAction = actionCollection()->addAction("history");
+    m_historyAction->setIcon(KIcon("view-history"));
+    m_historyAction->setText(i18nc("@action::inmenu", "History..."));
+    m_historyAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_H));
+    connect(m_historyAction, SIGNAL(triggered()), this, SLOT(showHistoryDialog()));
+
     QShortcut *shortcut = new QShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_M), this);
     connect(shortcut, SIGNAL(activated()), this, SLOT(easterEggTriggered()));
 
@@ -251,6 +276,12 @@ void MainWindow::checkForUpdates()
     m_backend->updateCache();
 }
 
+void MainWindow::downloadPackagesFromList()
+{
+    initDownloadWidget();
+    MuonMainWindow::downloadPackagesFromList();
+}
+
 void MainWindow::errorOccurred(QApt::ErrorCode error, const QVariantMap &details)
 {
     Q_UNUSED(details);
@@ -262,7 +293,6 @@ void MainWindow::errorOccurred(QApt::ErrorCode error, const QVariantMap &details
         if (m_downloadWidget) {
             m_downloadWidget->clear();
         }
-    case QApt::AuthError:
         m_managerWidget->setEnabled(true);
         QApplication::restoreOverrideCursor();
         returnFromPreview();
@@ -287,6 +317,7 @@ void MainWindow::workerEvent(QApt::WorkerEvent event)
     case QApt::CacheUpdateFinished:
     case QApt::CommitChangesFinished:
         reload();
+    case QApt::PackageDownloadFinished:
         returnFromPreview();
         if (m_warningStack.size() > 0) {
             showQueuedWarnings();
@@ -296,6 +327,9 @@ void MainWindow::workerEvent(QApt::WorkerEvent event)
             showQueuedErrors();
             m_errorStack.clear();
         }
+
+        m_downloadWidget->deleteLater();
+        m_downloadWidget = 0;
         break;
     case QApt::PackageDownloadStarted:
         if (m_downloadWidget) {
@@ -323,7 +357,6 @@ void MainWindow::workerEvent(QApt::WorkerEvent event)
                    m_statusWidget, SLOT(updateXapianProgress(int)));
         m_statusWidget->hideXapianProgress();
         break;
-    case QApt::PackageDownloadFinished:
     case QApt::InvalidEvent:
     default:
         break;
@@ -444,11 +477,9 @@ void MainWindow::setActionsEnabled(bool enabled)
         m_previewAction->setEnabled(changesPending);
     }
 
-    if (isConnected()) {
-        m_applyAction->setEnabled(changesPending);
-    } else {
-        m_applyAction->setEnabled(false);
-    }
+    m_downloadListAction->setEnabled(isConnected());
+
+    m_applyAction->setEnabled(changesPending);
 
     m_undoAction->setEnabled(!m_backend->isUndoStackEmpty());
     m_redoAction->setEnabled(!m_backend->isRedoStackEmpty());
@@ -478,6 +509,36 @@ void MainWindow::closeSettingsDialog()
     m_settingsDialog = 0;
 }
 
+void MainWindow::showHistoryDialog()
+{
+    if (!m_historyDialog) {
+        m_historyDialog = new KDialog(this);
+
+        KConfigGroup dialogConfig(KSharedConfig::openConfig("muonrc"),
+                                  "HistoryDialog");
+        m_historyDialog->restoreDialogSize(dialogConfig);
+
+        connect(m_historyDialog, SIGNAL(finished()), SLOT(closeHistoryDialog()));
+        HistoryView *historyView = new HistoryView(m_historyDialog);
+        m_historyDialog->setMainWidget(historyView);
+        m_historyDialog->setWindowTitle(i18nc("@title:window", "Package History"));
+        m_historyDialog->setWindowIcon(KIcon("view-history"));
+        m_historyDialog->setButtons(KDialog::Close);
+        m_historyDialog->show();
+    } else {
+        m_historyDialog->raise();
+    }
+}
+
+void MainWindow::closeHistoryDialog()
+{
+    KConfigGroup dialogConfig(KSharedConfig::openConfig("muonrc"),
+                              "HistoryDialog");
+    m_historyDialog->saveDialogSize(dialogConfig, KConfigBase::Persistent);
+    m_historyDialog->deleteLater();
+    m_historyDialog = 0;
+}
+
 void MainWindow::easterEggTriggered()
 {
     KDialog *dialog = new KDialog(this);
