@@ -28,6 +28,7 @@ class QSplitter;
 class QStackedWidget;
 class QToolBox;
 class KAction;
+class KDialog;
 
 class ManagerSettingsDialog;
 class FilterWidget;
@@ -64,11 +65,16 @@ private:
     KAction *m_autoRemoveAction;
     KAction *m_previewAction;
     KAction *m_applyAction;
+    KAction *m_historyAction;
     KAction *m_saveInstalledAction;
     KAction *m_saveSelectionsAction;
     KAction *m_loadSelectionsAction;
+    KAction *m_createDownloadListAction;
+    KAction *m_downloadListAction;
+    KAction *m_loadArchivesAction;
 
     ManagerSettingsDialog *m_settingsDialog;
+    KDialog *m_historyDialog;
     FilterWidget *m_filterBox;
     ManagerWidget *m_managerWidget;
     ReviewWidget *m_reviewWidget;
@@ -87,6 +93,7 @@ private Q_SLOTS:
     void markDistUpgrade();
     void markAutoRemove();
     void checkForUpdates();
+    void downloadPackagesFromList();
     void workerEvent(QApt::WorkerEvent event);
     void errorOccurred(QApt::ErrorCode error, const QVariantMap &details);
     void previewChanges();
@@ -102,6 +109,8 @@ public Q_SLOTS:
     void revertChanges();
     void editSettings();
     void closeSettingsDialog();
+    void showHistoryDialog();
+    void closeHistoryDialog();
 };
 
 #endif
