@@ -90,12 +90,29 @@ void UpdaterWindow::setupActions()
 {
     MuonMainWindow::setupActions();
 
+    m_createDownloadListAction = actionCollection()->addAction("save_download_list");
+    m_createDownloadListAction->setIcon(KIcon("document-save-as"));
+    m_createDownloadListAction->setText(i18nc("@action", "Save Package Download List..."));
+    connect(m_createDownloadListAction, SIGNAL(triggered()), this, SLOT(createDownloadList()));
+
+    m_downloadListAction = actionCollection()->addAction("download_from_list");
+    m_downloadListAction->setIcon(KIcon("download"));
+    m_downloadListAction->setText(i18nc("@action", "Download Packages From List..."));
+    connect(m_downloadListAction, SIGNAL(triggered()), this, SLOT(downloadPackagesFromList()));
+    if (!isConnected()) {
+        m_downloadListAction->setDisabled(false);
+    }
+    connect(this, SIGNAL(shouldConnect(bool)), m_downloadListAction, SLOT(setEnabled(bool)));
+
+    m_loadArchivesAction = actionCollection()->addAction("load_archives");
+    m_loadArchivesAction->setIcon(KIcon("document-open"));
+    m_loadArchivesAction->setText(i18nc("@action", "Add Downloaded Packages"));
+    connect(m_loadArchivesAction, SIGNAL(triggered()), this, SLOT(loadArchives()));
+
     m_applyAction = actionCollection()->addAction("apply");
     m_applyAction->setIcon(KIcon("dialog-ok-apply"));
     m_applyAction->setText(i18nc("@action Downloads and installs updates", "Install Updates"));
-    m_applyAction->setEnabled(isConnected());
     connect(m_applyAction, SIGNAL(triggered()), this, SLOT(startCommit()));
-    connect(this, SIGNAL(shouldConnect(bool)), m_applyAction, SLOT(setEnabled(bool)));
 
     m_revertAction = actionCollection()->addAction("revert");
     m_revertAction->setIcon(KIcon("document-revert"));
@@ -117,6 +134,12 @@ void UpdaterWindow::checkForUpdates()
     m_backend->updateCache();
 }
 
+void UpdaterWindow::downloadPackagesFromList()
+{
+    initDownloadWidget();
+    MuonMainWindow::downloadPackagesFromList();
+}
+
 void UpdaterWindow::workerEvent(QApt::WorkerEvent event)
 {
     MuonMainWindow::workerEvent(event);
@@ -132,6 +155,7 @@ void UpdaterWindow::workerEvent(QApt::WorkerEvent event)
     case QApt::CacheUpdateFinished:
     case QApt::CommitChangesFinished:
         reload();
+    case QApt::PackageDownloadFinished:
         returnFromPreview();
         if (m_warningStack.size() > 0) {
             showQueuedWarnings();
@@ -144,8 +168,7 @@ void UpdaterWindow::workerEvent(QApt::WorkerEvent event)
         break;
     case QApt::PackageDownloadStarted:
         if (m_downloadWidget) {
-            // TODO: Change to "Downloading Updates" after string freeze
-            m_downloadWidget->setHeaderText(i18nc("@info", "<title>Downloading Packages</title>"));
+            m_downloadWidget->setHeaderText(i18nc("@info", "<title>Downloading Updates</title>"));
             m_stack->setCurrentWidget(m_downloadWidget);
             connect(m_downloadWidget, SIGNAL(cancelDownload()), m_backend, SLOT(cancelDownload()));
         }
@@ -153,13 +176,11 @@ void UpdaterWindow::workerEvent(QApt::WorkerEvent event)
         break;
     case QApt::CommitChangesStarted:
         if (m_commitWidget) {
-            // TODO: Change to "Installing Updates" after string freeze
-            m_commitWidget->setHeaderText(i18nc("@info", "<title>Committing Changes</title>"));
+            m_commitWidget->setHeaderText(i18nc("@info", "<title>Installing Updates</title>"));
             m_stack->setCurrentWidget(m_commitWidget);
         }
         QApplication::restoreOverrideCursor();
         break;
-    case QApt::PackageDownloadFinished:
     case QApt::InvalidEvent:
     default:
         break;
@@ -175,7 +196,6 @@ void UpdaterWindow::errorOccurred(QApt::ErrorCode error, const QVariantMap &args
         if (m_downloadWidget) {
             m_downloadWidget->clear();
         }
-    case QApt::AuthError:
         m_updaterWidget->setEnabled(true);
         QApplication::restoreOverrideCursor();
         returnFromPreview();
@@ -210,6 +230,7 @@ void UpdaterWindow::initCommitWidget()
 void UpdaterWindow::startCommit()
 {
     setActionsEnabled(false);
+    m_updaterWidget->setEnabled(false);
     QApplication::setOverrideCursor(Qt::WaitCursor);
     initDownloadWidget();
     initCommitWidget();
@@ -255,12 +276,9 @@ void UpdaterWindow::setActionsEnabled(bool enabled)
         return;
     }
 
-    if (isConnected()) {
-        m_applyAction->setEnabled(m_backend->areChangesMarked());
-    } else {
-        m_applyAction->setEnabled(false);
-    }
+    m_downloadListAction->setEnabled(isConnected());
 
+    m_applyAction->setEnabled(m_backend->areChangesMarked());
     m_undoAction->setEnabled(!m_backend->isUndoStackEmpty());
     m_redoAction->setEnabled(!m_backend->isRedoStackEmpty());
     m_revertAction->setEnabled(!m_backend->isUndoStackEmpty());
