@@ -29,7 +29,7 @@
 static const char description[] =
     I18N_NOOP("An update manager");
 
-static const char version[] = "1.1.3 \"Bullish Bandril\"";
+static const char version[] = "1.1.65 \"Caustic Carrionite\"";
 
 int main(int argc, char **argv)
 {
@@ -37,7 +37,6 @@ int main(int argc, char **argv)
                      KAboutData::License_GPL, ki18n("©2010, 2011 Jonathan Thomas"), KLocalizedString(), 0);
     about.addAuthor(ki18n("Jonathan Thomas"), KLocalizedString(), "echidnaman@kubuntu.org");
     about.setProgramIconName("system-software-update");
-    about.setProductName("muon/updater");
 
     KCmdLineArgs::init(argc, argv, &about);
 
